import urllib2

from bs4 import BeautifulSoup


class Player(object):
    """A player that is useful for the informer."""
    def __init__(self, name, scores, retired=False):
        self.name = name
        self.scores = scores
        self.retired = retired

    def __unicode__(self):
        return "<P: %s (%s)>" % (self.name, self.scores)

    def repr(self):
        return repr(self.__unicode__())


def get_players(line1, line2):
    """Transform two game lines in Players."""
    stt1, name1, scores1 = line1
    stt2, name2, scores2 = line2

    if stt1 and stt2:
        raise ValueError("Duplicated status: stt1=%r  stt2=%r" % (stt1, stt2))

    game_stt = stt1 or stt2
    if game_stt != 'Fin.':
        # game not finished yet
        return

    print "======= l1", line1
    print "======= l2", line2
    print "==== Game, stt", game_stt
    # see who won
    processed_scores1 = [0] * 5
    processed_scores2 = [0] * 5
    w1 = w2 = 0
    for i, (sc1, sc2) in enumerate(zip(scores1, scores2)):
        if not sc1 and not sc2:
            break
        sc1, sc2 = int(sc1), int(sc2)
        assert sc1 != sc2

        processed_scores1[i] = sc1
        processed_scores2[i] = sc2
        if sc1 > sc2:
            w1 += 1
        else:
            w2 += 1

    if w1 > w2:
        p_winnr = Player(name1, scores1)
        p_loser = Player(name2, scores2)
    elif w1 < w2:
        p_winnr = Player(name2, scores2)
        p_loser = Player(name1, scores1)
    else:
        raise ValueError("Tied in sets? sc1=%r  sc2=%r" % (scores1,scores2))


    print "======= proc", processed_scores1, processed_scores2, w1, w2
    return p_winnr, p_loser



class Tournament(object):
    """The tournament in livescores.com."""

    def __init__(self, row):
        td1, td2 = row.find_all("td")
        self.name = td1.text.strip()

        gender, sdtype = td2.text.strip().split()
        self.gender = gender[0]
        self.sdtype = sdtype[0]

        self._half_game_line = None
        self.games = []

    def append(self, fds):
        """Append  some fds to game lines."""
        # get useful info
        assert len(fds) == 8
        texts = [fd.text.strip() for fd in fds]
        stt = texts[0]
        player_name = texts[1]
        assert texts[2] == u''
        scores = texts[3:]
        game_line = (stt, player_name, scores)

        # need two lines for a game
        if self._half_game_line is None:
            self._half_game_line = game_line
        else:
            players = get_players(self._half_game_line, game_line)
            self._half_game_line = None
            if players is not None:
                self.games.append(players)

    def __unicode__(self):
        return "<T: %s [%s/%s]>" % (self.name, self.gender, self.sdtype)

    def __repr__(self):
        return repr(self.__unicode__())


def get(raw=None):
    if raw is None:
        raw = urllib2.urlopen("http://www.livescores.com/tennis.dll").read()
    soup = BeautifulSoup(raw)
    tablas = soup.find("table")

    inner1 = list(tablas)[3]
    inner2 = inner1.find('table')
    rows = inner2.find('tr')
    inner3 = list(rows)[4]

    contenido = inner3.find('table')
    rows = [r for r in contenido.find_all("tr") if r.text]


    tournaments = []
    for row in rows:
        repr_row = repr(row)

        if 'class="title"' in repr_row:
            # title!
            t = Tournament(row)
            print "==== tournament:", t
            tournaments.append(t)
            continue

        if 'class="match-light"' in repr_row:
            # current hour and date
            continue

        inner_fds = row.find_all("td")
        if len(inner_fds):
            # game line!
            t.append(inner_fds)

    print "all processed!"
    for t in tournaments:
        if t.games:
            print "====  t", t, t.games


if  __name__ == '__main__':
    get()
